﻿using System.ComponentModel.DataAnnotations;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Todo.Core.Application.Services.UserConnectionStore;
using Todo.Infrastructure.UserConnectionStore;

namespace Todo.Infrastructure;

public static class DependencyInjection
{
    public static IServiceCollection AddInfrastructure(
        this IServiceCollection services,
        IConfiguration configuration)
    {
        var giteaAuthOptions = new GiteaAuthOptions();
        var giteaOptions = configuration.GetRequiredSection("GITEA");
        giteaOptions.Bind(giteaAuthOptions);

        services
            .AddOptions<GiteaAuthOptions>()
            .Bind(giteaOptions)
            .ValidateDataAnnotations();

        services
            .AddSingleton<IUserConnectionStore, InMemoryUserConnectionStore>();

        return services.AddAuthentication(
                options =>
                {
                    options.DefaultScheme = "Cookies";
                    options.DefaultChallengeScheme = "oidc";
                })
            .AddCookie("Cookies")
            .AddOpenIdConnect(
                "oidc",
                options =>
                {
                    options.Authority = giteaAuthOptions.Url;
                    options.ClientId = giteaAuthOptions.ClientId;
                    options.ClientSecret = giteaAuthOptions.ClientSecret;
                    options.ResponseType = "code";

                    options.SaveTokens = true;
                })
            .Services;
    }

    public static IApplicationBuilder UseInfrastructure(
        this IApplicationBuilder app)
        => app.UseCookiePolicy(
            new CookiePolicyOptions
            {
                Secure = CookieSecurePolicy.Always
            });
}

public class GiteaAuthOptions
{
    [Required]
    public string Url { get; set; }

    [Required]
    public string ClientId { get; init; }

    [Required]
    public string ClientSecret { get; init; }
}